/***************************************************************************
 *                                                                         *
 *  MAXIMUS-CBCS Source Code, Version 1.02                                 *
 *  Copyright 1989, 1990 by Scott J. Dudley.  All rights reserved.         *
 *                                                                         *
 *  Regular expression compiler.  See also STRSTRX.C                       *
 *                                                                         *
 *  For complete details of the licensing restrictions, please refer to    *
 *  the licence agreement, which is published in its entirety in MAX.C     *
 *  and LICENCE.MAX.                                                       *
 *                                                                         *
 *  USE OF THIS FILE IS SUBJECT TO THE RESTRICTIONS CONTAINED IN THE       *
 *  MAXIMUS-CBCS LICENSING AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF THIS *
 *  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES, OR IF YOU DO NOT HAVE   *
 *  THESE FILES, YOU SHOULD IMMEDIATELY CONTACT THE AUTHOR AT ONE OF THE   *
 *  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO USE THIS    *
 *  FILE WITHOUT HAVING ACCEPTED THE TERMS OF THE MAXIMUS-CBCS LICENSING   *
 *  AGREEMENT, OR SUCH OTHER AGREEMENT AS YOU ARE ABLE TO REACH WITH THE   *
 *  AUTHOR.                                                                *
 *                                                                         *
 *  You can contact the author at one of the address listed below:         *
 *                                                                         *
 *  Scott Dudley           FidoNet  1:249/106                              *
 *  777 Downing St.        IMEXnet  89:483/202                             *
 *  Kingston, Ont.         Internet f106.n249.z1.fidonet.org               *
 *  Canada - K7M 5N3       BBS      (613) 389-8315 - HST/14.4K, 24hrs      *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <mem.h>
#include "prog.h"
#include "alc.h"
#include "rep.h"

int pascal Compile_REP(char *exp,struct _rep *rep)
{
  char found,
       neg,
       start,
       end;

  int ch,
      x;

  for (ch=0;*exp && ch < MAX_REP;ch++)
  {
    rep->table[ch]=NULL;
    rep->type[ch]=0;

    switch(*exp)
    {
      case '^':
        rep->type[ch] |= CHAR_SOL;
        break;

      case '$':
        rep->type[ch] |= CHAR_EOL;
        break;

      case '.':
        rep->type[ch] |= CHAR_ANY;
        break;

      case '*':
        if (ch)
        {
          if (rep->type[ch-1] & (CHAR_ZMORE | CHAR_1MORE))
            return CERR_2R;

          rep->type[ch-1] |= CHAR_ZMORE;
        }
        else return CERR_NOREPEAT;

        ch--;
        break;

      case '+':
        if (ch)
        {
          if (rep->type[ch-1] & (CHAR_ZMORE | CHAR_1MORE))
            return CERR_2R;

          rep->type[ch-1] |= CHAR_1MORE;
        }
        else return CERR_NOREPEAT;

        ch--;
        break;

      case '[':   /* Start bracket search! */
        rep->type[ch] |= CHAR_TABLE;

        /* Allocate and set table to FALSE */
        if ((rep->table[ch]=(char *)malloc(TABLE_LEN))==NULL)
          return CERR_NOMEM;

        memset(rep->table[ch],'\x00',TABLE_LEN);

        if (*++exp=='^')
        {
          neg=TRUE;
          exp++;
        }
        else neg=FALSE;

        if (*exp==']')
          return CERR_EMPTYSET;

        found=FALSE;

        while (*exp)
        {
          if (*exp==']' && ((*(exp-1) != QUOTE) || (*(exp-2) == QUOTE)))
          {
            found=TRUE;
            break;
          }

          switch(*exp)
          {
            case '^':
              rep->type[ch] |= CHAR_SOL;
              break;

            case '$':
              rep->type[ch] |= CHAR_EOL;
              break;

            case QUOTE:
              if (! *++exp) /* If the next char is the end of string! */
                break;
              /* else fall-through */

            default:
              if (*(exp+1)=='-')
              {
                start=*exp;
                end=*(exp+=2);

                if (start < end)
                {
                  for (x=start;x <= end;x++)
                    rep->table[ch][x]=TRUE;
                }
                else
                {
                  for (x=end;x >= start;x--)
                    rep->table[ch][x]=TRUE;
                }
              }
              else
                rep->table[ch][*exp]=TRUE;
              break;
          }

          exp++;
        }

        if (! found)
          return CERR_NOTERM;

        if (neg)
          for (x=0;x < TABLE_LEN;x++)
            rep->table[ch][x]=(char)((rep->table[ch][x]==FALSE) ? TRUE : FALSE);
        break;

      case QUOTE:
        if (! *++exp) /* If the next char is the end of string! */
          return CERR_NOQUOTE;
        /* else fall-through */

      default:
        rep->type[ch] |= CHAR_NORM;
        rep->c[ch]=*exp;
        break;
    }

    exp++;
  }

  rep->max_ch=ch;

  if (ch==MAX_REP)
    return -1;
  else return 0;
}

